@@ -8,14 +8,14 @@ new (_x = /*#__PURE__*/new WeakMap(), _m = /*#__PURE__*/new WeakSet(), (_temp =
       writable: true,
       value: void 0
     }), babelHelpers.defineProperty(this, "x", void 0)), (() => {
-      hasX = o => _x.has(o);
-      hasM = o => _m.has(o);
+      hasX = o => _x.has(babelHelpers.checkInRHS(o));
+      hasM = o => _m.has(babelHelpers.checkInRHS(o));
     })(), _initClass();
   }
 }, (() => {
   class Foo {
     static m() {}
   }
-  [_Foo, _initClass] = babelHelpers.applyDecs(Foo, [], [dec]);
+  [_Foo, _initClass] = babelHelpers.applyDecs2203(Foo, [], [dec]);
 })(), _temp))();
 function _m2() {}
